<?php
    require_once KCONFIG . '/classes/classKYaml.php';
    require_once KCONFIG . '/classes/KGatewayHardwareInfo.php';

    class KGatewaySwitchYamlException extends KYamlException
    {
        /* VOID */
    }

    class KGatewaySwitchYamlVLANModeValidator implements KYamlValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'VLANMode';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            if( !in_array( $data, KGatewaySwitchYaml::$vlanmodes ) )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-invalid-vlan-mode' );
            }
        }
    }

    class KGatewaySwitchYamlVLANPortNameReferenceValidator implements KYamlValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'VLANPortNameReference';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            $info = KGatewayHardwareInfo::getInstance( );

            if( is_null( $info->getSwitchPortConfig( $data ) ) )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-invalid-vlan-mode' );
            }
        }
    }

    class KGatewaySwitchYamlVLANIdValidator implements KYamlValidator
    {
        /**
         * Return validator name
         *
         * @return string name
         */
        public function getName( )
        {
            return 'VLANId';
        }

        /**
         * Check if $data is valid and return true if it is, anything else
         * otherwise.
         * Throw exception in case of invalid data
         *
         * @param KYaml $yaml YAML object being validated
         * @param array $rules Current rule set (as in KYaml::validate)
         * @param mixed $data Current data (as in KYaml::validate)
         * @param string $path Parent
         */
        public function validate( KYaml $yaml, &$rule, &$data, &$path )
        {
            if( $data < 2 || $data > 3999 )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-restricted-vlan-id' );
            }

            $yamldoc = $yaml->getYaml();
            $counter = 0;
            foreach( $yamldoc['VLAN'] as $cfg )
            {
                if( $cfg['Id'] == $data )
                {
                    ++$counter;
                }
            }
            if( $counter > 1 )
            {
                throw new KGatewaySwitchYamlException( 'gateway-switch-error-duplicated-vlan-id' );
            }
        }
    }

    class KGatewaySwitchYaml extends KYaml
    {
        public static $vlanmodes = array( 'UNTAGGED', 'MEMBER' );
        const DEFAULT_VLAN_MODE = 0;
        // UNTAGGED

        public function __construct( )
        {
            //@formatter:off
            $this->rules = array(
                'VLAN' => array(
                    'type' => KYaml::TYPE_LIST,
                    'flags' => KYaml::FLAG_NORMALIZE | KYaml::FLAG_EMPTY,
                    'children' => array(
                        '?' => array(
                            'type' => KYaml::TYPE_LISTHASH,
                            'children' => array(
                                'PortName' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'validator' => 'VLANPortNameReference'
                                ), // VLAN.?.PortName
                                'Id' => array(
                                    'type' => KYaml::TYPE_INTEGER,
                                    'validator' => 'VLANId'
                                ), // VLAN.?.Id
                                'Mode' => array(
                                    'type' => KYaml::TYPE_STRING,
                                    'validator' => 'VLANMode'
                                ) // VLAN.?.Mode
                            )
                        ) // VLAN.?
                    )
                ) // VLAN
            );
            //@formatter:on
            $app = KSystem::getInstance( );
            parent::__construct( $app->getConfigDir( ) . DIRECTORY_SEPARATOR . 'kgateway-switch.yaml' );

            // validators
            $this->addValidator( new KGatewaySwitchYamlVLANModeValidator( ) );
            $this->addValidator( new KGatewaySwitchYamlVLANPortNameReferenceValidator( ) );
            $this->addValidator( new KGatewaySwitchYamlVLANIdValidator( ) );
        }

        ////////////////////////////////////////////////////////////////////////
        // VLAN maninpulation
        ////////////////////////////////////////////////////////////////////////

        /**
         * Get VLAN configuration array reference.
         *
         * @param string $name Port name
         * @return &array|null
         */
        protected function &_getVLANConfigurationByPortName( $name )
        {
            static $null = null;

            foreach( $this->yaml['VLAN'] as &$vlan )
            {
                if( $vlan['PortName'] === $name )
                {
                    return $vlan;
                }
            }

            return $null;
        }

        /**
         * Create configuration array for VLAN port.
         *
         * @param string $name Port name
         * @param string|int $id VLAN id
         * @param string $mode VLAN mode
         * @return array
         */
        protected function _createVLANConfiguration( $name, $id, $mode )
        {
            //@formatter:off
            return array(
                'PortName' => $name,
                'Id' => $id,
                'Mode' => $mode
            );
            //@formatter:on
        }

        /**
         * Get the VLAN configuration array, filtered by PortName. If does not
         * exists, then use hardware info to create a configuration using the
         * default VLAN id. If the hardware info does not exists, then this
         * method returns null.
         *
         * @param string $name Port name
         * @return array|null
         */
        public function getVLANConfigByPortName( $name )
        {
            $myconfig = &$this->_getVLANConfigurationByPortName( $name );
            if( !is_null( $myconfig ) )
            {
                return $myconfig;
            }

            $hwinfo = KGatewayHardwareInfo::getInstance( );
            $port = $hwinfo->getSwitchPortConfig( $name );
            if( !is_null( $port ) )
            {
                return $this->_createVLANConfiguration( $name, $port['DefaultVlanId'], self::$vlanmodes[self::DEFAULT_VLAN_MODE] );
            }

            return null;
        }

        /**
         * Set VLAN configuration.
         *
         * @param string $name Port name
         * @param string|int $id VLAN id
         * @param string $mode VLAN mode
         */
        public function setVLANConfig( $name, $id, $mode )
        {
            $myconfig = &$this->_getVLANConfigurationByPortName( $name );
            if( is_null( $myconfig ) )
            {
                $this->yaml['VLAN'][] = $this->_createVLANConfiguration( $name, $id, $mode );
            }
            else
            {
                $myconfig['Id'] = $id;
                $myconfig['Mode'] = $mode;
            }
        }
    }
